@@ -26,6 +26,7 @@
 #include <libgnome/gnome-program.h>
 #include <libgnome/gnome-i18n.h>
 #include <libgnomeui/gnome-ui-init.h>
+#include <libgnomeui/gnome-authentication-manager.h>
 #include <gtk/gtk.h>
 #include <bonobo/Bonobo.h>
 #include <bonobo/bonobo-main.h>
@@ -125,6 +126,8 @@ main (int argc, char **argv)
 				      GNOME_PARAM_APP_DATADIR, DATADIR,
 				      NULL);
 
+	gnome_authentication_manager_init ();
+
 	g_random_set_seed (time(0));
 
 #ifdef ENABLE_NLS
@@ -389,7 +392,14 @@ rb_handle_cmdline (char **argv, int argc,
 		tmp = rb_uri_resolve_relative (argv[i]);
 			
 		if (rb_uri_exists (tmp) == TRUE) {
-			GNOME_Rhythmbox_handleFile (shell, tmp, &ev);
+			GError *error = NULL;
+			char *utf8 = g_filename_to_utf8 (tmp, -1, NULL, NULL, &error);
+			if (!utf8 && error) {
+				g_error (error->message);
+				continue;
+			}
+ 			GNOME_Rhythmbox_handleFile (shell, utf8, &ev);
+			g_free (utf8);
 		}
 		
 		g_free (tmp);
