@@ -58,7 +58,7 @@
 #include "rb-iradio-source.h"
 #include "rb-library-source.h"
 #include "eel-gconf-extensions.h"
-
+#include "rb-util.h"
 #include "rb-play-order.h"
 
 #ifdef HAVE_XIDLE_EXTENSION
@@ -144,7 +144,8 @@ static void rb_shell_player_state_changed_cb (GConfClient *client,
 					      GConfEntry *entry,
 					      RBShellPlayer *playa);
 static void rb_shell_player_sync_volume (RBShellPlayer *player); 
-static void rb_shell_player_sync_replaygain (RBShellPlayer *player);
+static void rb_shell_player_sync_replaygain (RBShellPlayer *player,
+                                             RhythmDBEntry *entry);
 static void tick_cb (RBPlayer *player, long elapsed, gpointer data);
 static void eos_cb (RBPlayer *player, gpointer data);
 static void error_cb (RBPlayer *player, GError *err, gpointer data);
@@ -498,8 +499,8 @@ rb_shell_player_init (RBShellPlayer *player)
 	gtk_tooltips_enable (player->priv->tooltips);
 
 	/* Previous button */
-	image = gtk_image_new_from_stock (RB_STOCK_PREVIOUS,
-					  GTK_ICON_SIZE_LARGE_TOOLBAR);
+	image = rb_image_new_from_stock (RB_STOCK_PREVIOUS,
+					 GTK_ICON_SIZE_LARGE_TOOLBAR);
 
 	player->priv->prev_button = gtk_button_new ();
 	gtk_container_add (GTK_CONTAINER (player->priv->prev_button), image);
@@ -510,14 +511,14 @@ rb_shell_player_init (RBShellPlayer *player)
 			      _("Play previous song"), NULL);
 
 	/* Button images */
-	player->priv->play_image = gtk_image_new_from_stock (RB_STOCK_PLAY,
-							     GTK_ICON_SIZE_LARGE_TOOLBAR);
+	player->priv->play_image = rb_image_new_from_stock (RB_STOCK_PLAY,
+							    GTK_ICON_SIZE_LARGE_TOOLBAR);
 	g_object_ref (player->priv->play_image);
-	player->priv->pause_image = gtk_image_new_from_stock (RB_STOCK_PAUSE,
+	player->priv->pause_image = rb_image_new_from_stock (RB_STOCK_PAUSE,
 							     GTK_ICON_SIZE_LARGE_TOOLBAR);
 	g_object_ref (player->priv->pause_image);
-	player->priv->stop_image = gtk_image_new_from_stock (RB_STOCK_STOP,
-							     GTK_ICON_SIZE_LARGE_TOOLBAR);
+	player->priv->stop_image = rb_image_new_from_stock (RB_STOCK_STOP,
+							    GTK_ICON_SIZE_LARGE_TOOLBAR);
 	g_object_ref (player->priv->stop_image);
 
 	player->priv->play_pause_stop_button = gtk_button_new ();
@@ -528,8 +529,8 @@ rb_shell_player_init (RBShellPlayer *player)
 				  "clicked", G_CALLBACK (rb_shell_player_playpause), player);
 
 	/* Next button */
-	image = gtk_image_new_from_stock (RB_STOCK_NEXT,
-					  GTK_ICON_SIZE_LARGE_TOOLBAR);
+	image = rb_image_new_from_stock (RB_STOCK_NEXT,
+					 GTK_ICON_SIZE_LARGE_TOOLBAR);
 	player->priv->next_button = gtk_button_new ();
 	gtk_container_add (GTK_CONTAINER (player->priv->next_button), image);
 	g_signal_connect_swapped (G_OBJECT (player->priv->next_button),
@@ -718,8 +719,13 @@ rb_shell_player_get_property (GObject *object,
 		g_value_set_object (value, player->priv->tray_component);
 		break;
 	case PROP_PLAY_ORDER:
-		g_value_set_string_take_ownership (value, eel_gconf_get_string (CONF_STATE_PLAY_ORDER));
+	{
+		char *play_order = eel_gconf_get_string (CONF_STATE_PLAY_ORDER);
+		if (!play_order)
+			play_order = g_strdup ("linear");
+		g_value_set_string_take_ownership (value, play_order);
 		break;
+	}
 	case PROP_PLAYING:
 		g_value_set_boolean (value, rb_player_playing (player->priv->mmplayer));
 		break;
@@ -931,8 +937,10 @@ rb_shell_player_set_playing_entry (RBShellPlayer *player, RhythmDBEntry *entry)
 	songs = rb_source_get_entry_view (player->priv->source);
 
 	rb_shell_player_open_entry (player, entry, &error);
-	if (error == NULL)
+	if (error == NULL) {
+		rb_shell_player_sync_replaygain (player, entry);
 		rb_shell_player_play (player, &error);
+	}
 
 	if (error != NULL) {
 		rb_error_dialog (error->message);
@@ -943,8 +951,6 @@ rb_shell_player_set_playing_entry (RBShellPlayer *player, RhythmDBEntry *entry)
 	rb_entry_view_set_playing_entry (songs, entry);
 	rb_shell_player_sync_with_source (player);
 	rb_shell_player_sync_buttons (player);
-
-	rb_shell_player_sync_replaygain (player);
 }
 
 static void
@@ -965,7 +971,7 @@ rb_shell_player_get_playback_state (RBShellPlayer *player,
 
 	play_order = eel_gconf_get_string (CONF_STATE_PLAY_ORDER);
 	if (!play_order) {
-		g_critical (CONF_STATE_PLAY_ORDER " gconf key not found!");
+		g_warning (CONF_STATE_PLAY_ORDER " gconf key not found!");
 		return FALSE;
 	}
 
@@ -1011,7 +1017,7 @@ rb_shell_player_sync_play_order (RBShellPlayer *player)
 
 	if (!new_play_order) {
 		g_critical (CONF_STATE_PLAY_ORDER " gconf key not found!");
-		new_play_order = g_strdup ("");
+		new_play_order = g_strdup ("linear");
 	}
 
 	if (current_play_order == NULL
@@ -1054,11 +1060,7 @@ rb_shell_player_do_previous (RBShellPlayer *player)
 	RhythmDBEntry* entry;
 
 	if (player->priv->source != NULL) {
-
-		if (RB_IS_LIBRARY_SOURCE (player->priv->source)) {
-
-			rb_debug("previous algorithm for a library source");
-			
+		if (rb_source_can_pause (player->priv->source)) {
 			/* If we're in the first 2 seconds go to the previous song,
 			 * else restart the current one.
 			 */
@@ -1079,11 +1081,8 @@ rb_shell_player_do_previous (RBShellPlayer *player)
 				rb_player_set_time (player->priv->mmplayer, 0);
 				rb_header_sync_time (player->priv->header_widget);
 			}
-		}
-		else if (RB_IS_IRADIO_SOURCE (player->priv->source)) {
-		
-			rb_debug("previous algorithm for an iradio source");
-			
+		} else {
+			rb_debug("source can't pause, going to previous");
 			entry = rb_play_order_get_previous (player->priv->play_order);
 			
 			if (entry) {
@@ -1121,11 +1120,7 @@ rb_shell_player_auto_adjust_rating (RBShellPlayer *player, gboolean jumped)
 	entry_play_time = rb_shell_player_get_playing_time (player);
 	entry_duration = rb_shell_player_get_playing_song_duration (player);
 	entry_time_left = entry_duration - entry_play_time;
-	if (entry_time_left <= 0)
-		return;
-	
-	/* sloppy end of song */
-	if (entry_time_left < 16)
+	if (entry_time_left < 0)
 		entry_time_left = 0;
 	
 	if (entry_time_left == 0) {
@@ -1360,20 +1355,18 @@ rb_shell_player_sync_volume (RBShellPlayer *player)
 	rb_player_set_volume (player->priv->mmplayer,
 					volume);
 					
-	rb_shell_player_sync_replaygain (player);					
+	rb_shell_player_sync_replaygain (player, 
+					 rb_shell_player_get_playing_entry (player));					
 }
 
 static void
-rb_shell_player_sync_replaygain (RBShellPlayer *player)
+rb_shell_player_sync_replaygain (RBShellPlayer *player, RhythmDBEntry *entry)
 {
-	RhythmDBEntry *entry;
 	double entry_track_gain = 0;
 	double entry_track_peak = 0;
 	double entry_album_gain = 0;
 	double entry_album_peak = 0;
 	
-	entry = rb_shell_player_get_playing_entry (player);
-
 	if (entry != NULL) {
 		rhythmdb_read_lock (player->priv->db);
              	entry_track_gain = rhythmdb_entry_get_double (player->priv->db,
