@@ -128,12 +128,6 @@ static void rb_shell_corba_set_playing_time (PortableServer_Servant _servant,
 						   CORBA_long time, CORBA_Environment *ev);
 
 static Bonobo_PropertyBag rb_shell_corba_get_player_properties (PortableServer_Servant _servant, CORBA_Environment *ev);
-static void rb_shell_db_changed_cb (RhythmDB *db, RhythmDBEntry *entry,
-				    RBShell *shell);
-static void rb_shell_db_changed_prop_cb (RhythmDB *db, RhythmDBEntry *entry,
-					 RhythmDBPropType prop, const GValue *old,
-					 const GValue *new, RBShell *shell);
-
 
 static gboolean rb_shell_window_state_cb (GtkWidget *widget,
 					  GdkEvent *event,
@@ -328,7 +322,6 @@ struct RBShellPrivate
 
 	gboolean play_queued;
 
-	gboolean db_dirty;
 	guint async_state_save_id;
 
 	gboolean shutting_down;
@@ -614,11 +607,11 @@ rb_shell_sync_state (RBShell *shell)
 	}
 	
 	rb_debug ("saving playlists");
-	rb_playlist_manager_save_playlists (shell->priv->playlist_manager);
+	rb_playlist_manager_save_playlists (shell->priv->playlist_manager, TRUE);
 
 	rb_debug ("saving db");
 	rhythmdb_read_lock (shell->priv->db);
-	rhythmdb_save (shell->priv->db);
+	rhythmdb_save_blocking (shell->priv->db);
 	rhythmdb_read_unlock (shell->priv->db);
 	return FALSE;
 }
@@ -634,19 +627,22 @@ idle_save_state (RBShell *shell)
 	return FALSE;
 }
 
-static void
-rb_shell_db_changed_cb (RhythmDB *db, RhythmDBEntry *entry,
-			RBShell *shell)
+static gboolean
+idle_save_rhythmdb (RhythmDB *db)
 {
-	shell->priv->db_dirty = TRUE;
+	rhythmdb_read_lock (db);
+	rhythmdb_save (db);
+	rhythmdb_read_unlock (db);
+	
+	return FALSE;
 }
 
-static void
-rb_shell_db_changed_prop_cb (RhythmDB *db, RhythmDBEntry *entry,
-			     RhythmDBPropType prop, const GValue *old, const GValue *new,
-			     RBShell *shell)
+static gboolean
+idle_save_playlist_manager (RBPlaylistManager *mgr)
 {
-	rb_shell_db_changed_cb (db, entry, shell);
+	rb_playlist_manager_save_playlists (mgr, FALSE);
+
+	return TRUE;
 }
 
 static void
@@ -766,7 +762,7 @@ idle_do_action (struct RBShellAction *data)
 		g_timeout_add (500 + data->tries*200, (GSourceFunc) idle_do_action, data);
 		goto out_unlock;
 	} else
-		g_warning ("No entry %s in db", data->uri);
+		rb_debug ("No entry %s in db", data->uri);
 
 	data->shell->priv->play_queued = FALSE;
 
@@ -847,12 +843,15 @@ rb_shell_corba_handle_file (PortableServer_Servant _servant,
 	if (rb_playlist_can_handle (uri)) {
 		rb_debug ("parsing uri as playlist: %s", uri);
 		uri = rb_playlist_manager_parse_file (shell->priv->playlist_manager, uri);
+		if (!uri)
+			goto out;
 	} else {
 		rb_debug ("async adding uri: %s", uri);
 		rhythmdb_add_uri_async (shell->priv->db, uri);
 	}
 
 	rb_shell_queue_play (shell, uri);
+out:
 	g_object_unref (G_OBJECT (parser));
 }
 
@@ -958,16 +957,18 @@ shell_pb_get_prop (BonoboPropertyBag *bag,
 
 	case PROP_SHUFFLE:
 	{
-		char *play_order;
-		g_object_get (G_OBJECT (player), "play-order", &play_order, NULL);
-		BONOBO_ARG_SET_BOOLEAN (arg, !strcmp ("shuffle", play_order));
+		gboolean shuffle, repeat;
+		rb_shell_player_get_playback_state (player,
+						    &shuffle, &repeat);
+		BONOBO_ARG_SET_BOOLEAN (arg, shuffle);
 		break;
 	}
 
 	case PROP_SONG: {
 		GNOME_Rhythmbox_SongInfo *ret_val;
+		
 		ret_val = get_song_info_from_player (shell);
-		(GNOME_Rhythmbox_SongInfo*)arg->_value = ret_val;
+		arg->_value = (gpointer)ret_val;
 		if (ret_val == NULL) {
 			arg->_type = TC_null;
 		} else {
@@ -989,6 +990,11 @@ shell_pb_set_prop (BonoboPropertyBag *bag,
 		   CORBA_Environment *ev,
 		   gpointer           user_data)
 {
+	RBShell *shell = RB_SHELL (user_data);
+	RBShellPlayer *player;
+
+	player = RB_SHELL_PLAYER (shell->priv->player_shell);
+	
 	switch (arg_id) {
 
 	case PROP_VISIBILITY:
@@ -998,6 +1004,22 @@ shell_pb_set_prop (BonoboPropertyBag *bag,
 		bonobo_exception_set (ev, ex_Bonobo_PropertyBag_ReadOnly);
 		break;
 
+	case PROP_SHUFFLE:
+	{
+		gboolean repeat;
+		gboolean shuffle;
+
+
+		rb_shell_player_get_playback_state (player, &shuffle,
+						    &repeat);
+		shuffle = BONOBO_ARG_GET_BOOLEAN (arg);
+
+		rb_shell_player_set_playback_state (player,
+						    shuffle,
+						    repeat);
+		break;
+	}
+
 	default:
 		bonobo_exception_set (ev, ex_Bonobo_PropertyBag_NotFound);
 		break;
@@ -1144,8 +1166,12 @@ rb_shell_entry_changed_cb (GObject *object, GParamSpec *pspec, RBShell *shell)
 	
 	g_assert (strcmp (pspec->name, "playing-entry") == 0);
 	song_info = get_song_info_from_player (shell);
+	if (!song_info) {
+		rb_debug ("no song info returned!");
+		return;
+	}
 	arg = bonobo_arg_new (TC_GNOME_Rhythmbox_SongInfo);
-	(GNOME_Rhythmbox_SongInfo*)arg->_value = song_info;
+	arg->_value = (gpointer)song_info;
 	shell_notify_pb_changes (shell, "song", arg);
 	/* FIXME: arg should be released somehow */
 
@@ -1191,15 +1217,15 @@ rb_shell_construct (RBShell *shell)
 
 	shell->priv->window = GTK_WIDGET (win);
 
-	g_signal_connect (G_OBJECT (win), "window-state-event",
-			  G_CALLBACK (rb_shell_window_state_cb),
-			  shell);
-	g_signal_connect (G_OBJECT (win), "configure-event",
-			  G_CALLBACK (rb_shell_window_state_cb),
-			  shell);
-	g_signal_connect (G_OBJECT (win), "delete_event",
-			  G_CALLBACK (rb_shell_window_delete_cb),
-			  shell);
+	g_signal_connect_object (G_OBJECT (win), "window-state-event",
+				 G_CALLBACK (rb_shell_window_state_cb),
+				 shell, 0);
+	g_signal_connect_object (G_OBJECT (win), "configure-event",
+				 G_CALLBACK (rb_shell_window_state_cb),
+				 shell, 0);
+	g_signal_connect_object (G_OBJECT (win), "delete_event",
+				 G_CALLBACK (rb_shell_window_delete_cb),
+				 shell, 0);
   
 	rb_debug ("shell: creating container area");
 	shell->priv->container = bonobo_window_get_ui_container (win);
@@ -1247,23 +1273,10 @@ rb_shell_construct (RBShell *shell)
 			g_signal_connect_object (G_OBJECT (shell->priv->db), "load-complete",
 						 G_CALLBACK (rb_shell_load_complete_cb), shell,
 						 0);
-			rhythmdb_load (shell->priv->db);
 		} else {
 			shell->priv->load_complete = TRUE;
 		}
 	}
-	g_signal_connect_object (G_OBJECT (shell->priv->db),
-				 "entry_added",
-				 G_CALLBACK (rb_shell_db_changed_cb),
-				 shell, 0);
-	g_signal_connect_object (G_OBJECT (shell->priv->db),
-				 "entry_changed",
-				 G_CALLBACK (rb_shell_db_changed_prop_cb),
-				 shell, 0);
-	g_signal_connect_object (G_OBJECT (shell->priv->db),
-				 "entry_deleted",
-				 G_CALLBACK (rb_shell_db_changed_cb),
-				 shell, 0);
 
 	rb_debug ("shell: setting up tray icon");
 	tray_deleted_cb (NULL, NULL, shell);
@@ -1286,26 +1299,26 @@ rb_shell_construct (RBShell *shell)
 		shell->priv->player_shell = rb_shell_player_new (shell->priv->ui_component,
 								 tray_component);
 	}
-	g_signal_connect (G_OBJECT (shell->priv->player_shell), 
-			  "notify::repeat", 
-			  G_CALLBACK (rb_shell_property_changed_generic_cb), 
-			  shell);
-	g_signal_connect (G_OBJECT (shell->priv->player_shell), 
-			  "notify::shuffle", 
-			  G_CALLBACK (rb_shell_property_changed_generic_cb), 
-			  shell);
-	g_signal_connect (G_OBJECT (shell->priv->player_shell), 
-			  "notify::playing", 
-			  G_CALLBACK (rb_shell_property_changed_generic_cb), 
-			  shell);
-	g_signal_connect (G_OBJECT (shell->priv->player_shell),
-			  "window_title_changed",
-			  G_CALLBACK (rb_shell_player_window_title_changed_cb),
-			  shell);
-	g_signal_connect (G_OBJECT (shell->priv->player_shell),
-			  "duration_changed",
-			  G_CALLBACK (rb_shell_player_duration_changed_cb),
-			  shell);
+	g_signal_connect_object (G_OBJECT (shell->priv->player_shell), 
+				 "notify::repeat", 
+				 G_CALLBACK (rb_shell_property_changed_generic_cb), 
+				 shell, 0);
+	g_signal_connect_object (G_OBJECT (shell->priv->player_shell), 
+				 "notify::shuffle", 
+				 G_CALLBACK (rb_shell_property_changed_generic_cb), 
+				 shell, 0);
+	g_signal_connect_object (G_OBJECT (shell->priv->player_shell), 
+				 "notify::playing", 
+				 G_CALLBACK (rb_shell_property_changed_generic_cb), 
+				 shell, 0);
+	g_signal_connect_object (G_OBJECT (shell->priv->player_shell),
+				 "window_title_changed",
+				 G_CALLBACK (rb_shell_player_window_title_changed_cb),
+				 shell, 0);
+	g_signal_connect_object (G_OBJECT (shell->priv->player_shell),
+				 "duration_changed",
+				 G_CALLBACK (rb_shell_player_duration_changed_cb),
+				 shell, 0);
 	shell->priv->clipboard_shell = rb_shell_clipboard_new (shell->priv->ui_component,
 							       shell->priv->db);
 	shell->priv->source_header = rb_source_header_new (shell->priv->ui_component);
@@ -1313,12 +1326,12 @@ rb_shell_construct (RBShell *shell)
 	shell->priv->paned = gtk_hpaned_new ();
 
 	shell->priv->sourcelist = rb_sourcelist_new ();
-	g_signal_connect (G_OBJECT (shell->priv->sourcelist), "drop_received",
-			  G_CALLBACK (sourcelist_drag_received_cb), shell);
-	g_signal_connect (G_OBJECT (shell->priv->sourcelist), "source_activated",
-			  G_CALLBACK (source_activated_cb), shell);
-	g_signal_connect (G_OBJECT (shell->priv->sourcelist), "show_popup",
-			  G_CALLBACK (rb_shell_show_popup_cb), shell);
+	g_signal_connect_object (G_OBJECT (shell->priv->sourcelist), "drop_received",
+				 G_CALLBACK (sourcelist_drag_received_cb), shell, 0);
+	g_signal_connect_object (G_OBJECT (shell->priv->sourcelist), "source_activated",
+				 G_CALLBACK (source_activated_cb), shell, 0);
+	g_signal_connect_object (G_OBJECT (shell->priv->sourcelist), "show_popup",
+				 G_CALLBACK (rb_shell_show_popup_cb), shell, 0);
 
 	shell->priv->statusbar = rb_statusbar_new (shell->priv->db,
 						   shell->priv->ui_component,
@@ -1327,28 +1340,24 @@ rb_shell_construct (RBShell *shell)
 	rb_sourcelist_set_dnd_targets (RB_SOURCELIST (shell->priv->sourcelist), target_table,
 				       G_N_ELEMENTS (target_table));
 
-	g_signal_connect (G_OBJECT (shell->priv->sourcelist), "selected",
-			  G_CALLBACK (source_selected_cb), shell);
+	g_signal_connect_object (G_OBJECT (shell->priv->sourcelist), "selected",
+				 G_CALLBACK (source_selected_cb), shell, 0);
 
 	vbox = gtk_vbox_new (FALSE, 4);
 	shell->priv->notebook = gtk_notebook_new ();
 	gtk_notebook_set_show_tabs (GTK_NOTEBOOK (shell->priv->notebook), FALSE);
 	gtk_notebook_set_show_border (GTK_NOTEBOOK (shell->priv->notebook), FALSE);
-	g_signal_connect (G_OBJECT (shell->priv->notebook),
-			  "size_allocate",
-			  G_CALLBACK (paned_size_allocate_cb),
-			  shell);
+	g_signal_connect_object (G_OBJECT (shell->priv->notebook),
+				 "size_allocate",
+				 G_CALLBACK (paned_size_allocate_cb),
+				 shell, 0);
 
 	gtk_box_pack_start (GTK_BOX (vbox), GTK_WIDGET (shell->priv->source_header), FALSE, TRUE, 0);
 	gtk_box_pack_start_defaults (GTK_BOX (vbox), shell->priv->notebook);
 
-	if (gtk_widget_get_default_direction () != GTK_TEXT_DIR_RTL) {
-		gtk_paned_pack1 (GTK_PANED (shell->priv->paned), shell->priv->sourcelist, TRUE, TRUE);
-		gtk_paned_pack2 (GTK_PANED (shell->priv->paned), vbox, TRUE, TRUE);
-	} else {
-		gtk_paned_pack1 (GTK_PANED (shell->priv->paned), vbox, TRUE, TRUE);
-		gtk_paned_pack2 (GTK_PANED (shell->priv->paned), shell->priv->sourcelist, TRUE, TRUE);
-	}
+	gtk_paned_pack1 (GTK_PANED (shell->priv->paned), 
+			 shell->priv->sourcelist, TRUE, TRUE);
+	gtk_paned_pack2 (GTK_PANED (shell->priv->paned), vbox, TRUE, TRUE);
 
 	vbox = gtk_vbox_new (FALSE, 5);
 	gtk_container_set_border_width (GTK_CONTAINER (vbox), 5);
@@ -1384,11 +1393,11 @@ rb_shell_construct (RBShell *shell)
 	shell->priv->show_db_errors = FALSE;
 	gtk_widget_hide (shell->priv->load_error_dialog);
 
-	g_signal_connect (G_OBJECT (shell->priv->db), "error",
-			  G_CALLBACK (rb_shell_db_error_cb), shell);
+	g_signal_connect_object (G_OBJECT (shell->priv->db), "error",
+				 G_CALLBACK (rb_shell_db_error_cb), shell, 0);
 
-	g_signal_connect (G_OBJECT (shell->priv->load_error_dialog), "response",
-			  G_CALLBACK (rb_shell_load_failure_dialog_response_cb), shell);
+	g_signal_connect_object (G_OBJECT (shell->priv->load_error_dialog), "response",
+				 G_CALLBACK (rb_shell_load_failure_dialog_response_cb), shell, 0);
 
 	/* initialize sources */
 	while (known_sources[i] != NULL) {
@@ -1420,13 +1429,15 @@ rb_shell_construct (RBShell *shell)
 								 RB_LIBRARY_SOURCE (library_source),
 								 RB_IRADIO_SOURCE (iradio_source));
 
-	g_signal_connect (G_OBJECT (shell->priv->playlist_manager), "playlist_added",
-			  G_CALLBACK (rb_shell_playlist_added_cb), shell);
-	g_signal_connect (G_OBJECT (shell->priv->playlist_manager), "load_start",
-			  G_CALLBACK (rb_shell_playlist_load_start_cb), shell);
-	g_signal_connect (G_OBJECT (shell->priv->playlist_manager), "load_finish",
-			  G_CALLBACK (rb_shell_playlist_load_finish_cb), shell);
+	g_signal_connect_object (G_OBJECT (shell->priv->playlist_manager), "playlist_added",
+				 G_CALLBACK (rb_shell_playlist_added_cb), shell, 0);
+	g_signal_connect_object (G_OBJECT (shell->priv->playlist_manager), "load_start",
+				 G_CALLBACK (rb_shell_playlist_load_start_cb), shell, 0);
+	g_signal_connect_object (G_OBJECT (shell->priv->playlist_manager), "load_finish",
+				 G_CALLBACK (rb_shell_playlist_load_finish_cb), shell, 0);
 
+	g_timeout_add (10000, (GSourceFunc) idle_save_playlist_manager, shell->priv->playlist_manager);
+		
 	rb_shell_sync_window_state (shell);
 
 	bonobo_ui_component_thaw (shell->priv->ui_component, NULL);
@@ -1447,8 +1458,8 @@ rb_shell_construct (RBShell *shell)
 		else
 			rb_debug("CD is not available");
 
-		g_signal_connect (G_OBJECT (shell->priv->cd), "cd_changed",
-				  G_CALLBACK (audiocd_changed_cb), shell);
+		g_signal_connect_object (G_OBJECT (shell->priv->cd), "cd_changed",
+					 G_CALLBACK (audiocd_changed_cb), shell, 0);
         }
 	else
 		rb_debug ("No AudioCD device is available!");
@@ -1483,13 +1494,25 @@ rb_shell_construct (RBShell *shell)
 	}
 
 	/* GO GO GO! */
+	if (rhythmdb_exists) {
+		rb_debug ("loading database");
+		rhythmdb_load (shell->priv->db);
+/* Disabled for now */
+#if 0
+		rb_debug ("adding db save-when-needed thread");
+		g_timeout_add (10000, (GSourceFunc) idle_save_rhythmdb, shell->priv->db);
+#endif
+	}
+	
 	rb_debug ("shell: syncing window state");
 	rb_shell_sync_paned (shell);
 	gtk_widget_show_all (GTK_WIDGET (shell->priv->tray_icon));
 
 	/* Stop here if this is the first time. */
 	if (!eel_gconf_get_boolean (CONF_FIRST_TIME)) {
-		RBDruid *druid = rb_druid_new (shell->priv->db);
+		RBDruid *druid;
+		shell->priv->show_db_errors = TRUE;
+		druid = rb_druid_new (shell->priv->db);
 		gtk_widget_hide (GTK_WIDGET (shell->priv->window));
 		rb_druid_show (druid);
 		g_object_unref (G_OBJECT (druid));
@@ -1688,8 +1711,8 @@ rb_shell_append_source (RBShell *shell,
 	shell->priv->sources
 		= g_list_append (shell->priv->sources, source);
 
-	g_signal_connect (G_OBJECT (source), "deleted",
-			  G_CALLBACK (rb_shell_source_deleted_cb), shell);
+	g_signal_connect_object (G_OBJECT (source), "deleted",
+				 G_CALLBACK (rb_shell_source_deleted_cb), shell, 0);
 
 	gtk_notebook_append_page (GTK_NOTEBOOK (shell->priv->notebook),
 				  GTK_WIDGET (source),
@@ -1781,8 +1804,8 @@ rb_shell_select_source_internal (RBShell *shell,
 	shell->priv->selected_source = source;
 	
 	view = rb_source_get_entry_view (shell->priv->selected_source);
-	g_signal_connect (view, "notify::playing-entry", 
-			  G_CALLBACK(rb_shell_entry_changed_cb), shell);
+	g_signal_connect_object (view, "notify::playing-entry", 
+				 G_CALLBACK(rb_shell_entry_changed_cb), shell, 0);
 
 	/* show source */
 	gtk_notebook_set_current_page (GTK_NOTEBOOK (shell->priv->notebook),
@@ -2023,8 +2046,7 @@ rb_shell_cmd_contents (BonoboUIComponent *component,
 	gnome_help_display ("rhythmbox.xml", NULL, &error);
 
 	if (error != NULL) {
-		g_warning (error->message);
-
+		rb_error_dialog ("%s", error->message);
 		g_error_free (error);
 	}
 }
@@ -2052,6 +2074,11 @@ ask_file_response_cb (GtkDialog *dialog,
 #ifndef HAVE_GTK_2_3
 	char **files, **filecur;
 
+	if (response_id != GTK_RESPONSE_OK) {
+		gtk_widget_destroy (GTK_WIDGET (dialog));
+		return;
+	}
+
 	files = gtk_file_selection_get_selections (GTK_FILE_SELECTION (dialog));
 
 	gtk_widget_destroy (GTK_WIDGET (dialog));
@@ -2083,11 +2110,14 @@ ask_file_response_cb (GtkDialog *dialog,
 	shell->priv->show_db_errors = TRUE;
     
 	while (*filecur != NULL) {
-		if (g_utf8_validate (*filecur, -1, NULL)) {
+		char *utf8ized = g_filename_to_utf8 (*filecur, -1,
+						     NULL, NULL, NULL);
+		if (utf8ized) {
 			char *uri = gnome_vfs_get_uri_from_local_path (*filecur);
 			rhythmdb_add_uri_async (shell->priv->db, uri);
 			g_free (uri);
 		}
+		g_free (utf8ized);
 		filecur++;
 	}
 
@@ -2122,6 +2152,7 @@ ask_file_response_cb (GtkDialog *dialog,
 	g_free (current_dir);
 	gtk_widget_destroy (GTK_WIDGET (dialog));
 #endif
+	g_timeout_add (10000, (GSourceFunc) idle_save_rhythmdb, shell->priv->db);
 }
 
 static void
@@ -2133,15 +2164,15 @@ rb_shell_cmd_add_to_library (BonoboUIComponent *component,
 	GtkWidget *dialog;
     
 	stored = eel_gconf_get_string (CONF_STATE_ADD_DIR);
-	dialog = rb_ask_file_multiple (_("Choose Files or Directory"),
+	dialog = rb_ask_dir_multiple (_("Choose Files or Directory"),
 				      stored,
 			              GTK_WINDOW (shell->priv->window));
 	g_free (stored);
 
-	g_signal_connect (G_OBJECT (dialog),
-			  "response",
-			  G_CALLBACK (ask_file_response_cb),
-				shell);
+	g_signal_connect_object (G_OBJECT (dialog),
+				 "response",
+				 G_CALLBACK (ask_file_response_cb),
+				 shell, 0);
 }
 
 static void
@@ -2201,17 +2232,26 @@ rb_shell_quit (RBShell *shell)
 	bonobo_object_unref (BONOBO_OBJECT (shell));
 }
 
-static void
-rb_shell_load_complete_cb (RhythmDB *db, RBShell *shell)
+static gboolean
+idle_handle_load_complete (RBShell *shell)
 {
-	rb_debug ("load complete");
 	GDK_THREADS_ENTER ();
 
+	rb_debug ("load complete");
+
 	rb_playlist_manager_load_playlists (shell->priv->playlist_manager);
 	rb_shell_sync_selected_source (shell);
 	shell->priv->load_complete = TRUE;
 
 	GDK_THREADS_LEAVE ();
+
+	return FALSE;
+}
+
+static void
+rb_shell_load_complete_cb (RhythmDB *db, RBShell *shell)
+{
+	g_idle_add ((GSourceFunc) idle_handle_load_complete, shell);
 }
 
 static void
@@ -2353,6 +2393,7 @@ rb_shell_sync_selected_source (RBShell *shell)
 		const char *tmpname;
 		g_object_get (G_OBJECT (tmp->data), "internal-name", &tmpname, NULL);
 		if (!strcmp (internalname, tmpname)) {
+			g_assert (tmp->data != NULL);
 			rb_shell_select_source_internal (shell, tmp->data);
 			return;
 		}
@@ -2501,8 +2542,8 @@ tray_deleted_cb (GtkWidget *win, GdkEventAny *event, RBShell *shell)
 						   shell->priv->ui_component,
 						   shell->priv->db,
 						   GTK_WINDOW (shell->priv->window));
-	g_signal_connect (G_OBJECT (shell->priv->tray_icon), "delete_event",
-			  G_CALLBACK (tray_deleted_cb), shell);
+	g_signal_connect_object (G_OBJECT (shell->priv->tray_icon), "delete_event",
+				 G_CALLBACK (tray_deleted_cb), shell, 0);
 }
 
 static void 
@@ -2534,13 +2575,13 @@ rb_shell_session_init (RBShell *shell)
 
         client = gnome_master_client ();
 
-        g_signal_connect (G_OBJECT (client), 
-                          "save_yourself",
-                          G_CALLBACK (save_yourself_cb),
-                          shell);
+        g_signal_connect_object (G_OBJECT (client), 
+				 "save_yourself",
+				 G_CALLBACK (save_yourself_cb),
+				 shell, 0);
 
-        g_signal_connect (G_OBJECT (client),
-                          "die",
-                          G_CALLBACK (session_die_cb),
-                          shell);
+        g_signal_connect_object (G_OBJECT (client),
+				 "die",
+				 G_CALLBACK (session_die_cb),
+				 shell, 0);
 }
